//www.EricBogatin.com
// basic data acquisition features
// uses the Arduino Due and Digilent Analog Shield


#include <SPI.h>
#include <analogShield.h>
#include <math.h>

boolean flag_PLX = true; // for makerPlot, set this false, set for normal output no headers

long ibaudRate = 15000;  // 1500 for array into excel,
//long ibaudRate = 19200;  // 1500 for array into excel,
//long ibaudRate = 115200; //115200 for serial moinitor

long npts2meas = 1000;
long npts2ave = 1;
long nptsChan = 4;

float time2ave_sec = 5.; // the time to average in slow mode

//general purpose time measurements
unsigned long itime0_us = 0;
unsigned long itime1_us = 0;
unsigned long itime2_us = 0;
unsigned long itime_interval_us = 0;
float time_operation_us;

//slow mode variables
long iChan_2meas = 0; //channel to measure
float arrV_meas_nChan_slow_v[10];
float arrV_meas_nChan_DMM_v[10];


long itime2ave_ms;

//fast mode array variables
long arrV_meas_nChan_adu[10][1001]; // has the raw measured values fro n channels
long arrTime_us[1001];     // really an integer array
long iArray_counter = 0;

//Calibration info for digilent analog shield (DAS)
float arrCal_DAS_lo_adu[10];  //lo adu values in cal set up
float arrCal_DAS_hi_adu[10];  //hi adu values in cal set up
float cal_Vref_lo; //  the set voltage used to calibrate lo
float cal_Vref_hi; //  the set voltage used to calibrate hi
float arrCal_DAS_VperADU[10];    //calibration slope
float arrCal_DAS_V0[4];    //calibration V0 value when adu value = 0
long  iCal_chan_adu[10];


void setup() {
  // put your setup code here, to run once:

  cal_Vref_lo = 0.;
  arrCal_DAS_lo_adu[0] = 32743.; // the zero value for chan0
  arrCal_DAS_lo_adu[1] = 32815.; // the zero value for chan1
  arrCal_DAS_lo_adu[2] = 32785.; // the zero value for chan2
  arrCal_DAS_lo_adu[3] = 32816.; // the zero value for chan3
  arrCal_DAS_lo_adu[4] = 0.; // the zero value for chan4

  cal_Vref_hi = 2.500;
  arrCal_DAS_hi_adu[0] = 48952.; // the zero value for chan0
  arrCal_DAS_hi_adu[1] = 48853.; // the zero value for chan1
  arrCal_DAS_hi_adu[2] = 48939.; // the zero value for chan2
  arrCal_DAS_hi_adu[3] = 48864.; // the zero value for chan3
  arrCal_DAS_hi_adu[4] = 3113.; // the zero value for chan4

  arrCal_DAS_VperADU[0] = (cal_Vref_hi - cal_Vref_lo) / (arrCal_DAS_hi_adu[0] - arrCal_DAS_lo_adu[0]) ;
  arrCal_DAS_VperADU[1] = (cal_Vref_hi - cal_Vref_lo) / (arrCal_DAS_hi_adu[1] - arrCal_DAS_lo_adu[1]);
  arrCal_DAS_VperADU[2] = (cal_Vref_hi - cal_Vref_lo) / (arrCal_DAS_hi_adu[2] - arrCal_DAS_lo_adu[2]);
  arrCal_DAS_VperADU[3] = (cal_Vref_hi - cal_Vref_lo) / (arrCal_DAS_hi_adu[3] - arrCal_DAS_lo_adu[3]);
  arrCal_DAS_VperADU[4] = (cal_Vref_hi - cal_Vref_lo) / (arrCal_DAS_hi_adu[4] - arrCal_DAS_lo_adu[4]);

  arrCal_DAS_V0[0] = cal_Vref_hi - arrCal_DAS_VperADU[0] * arrCal_DAS_hi_adu[0];
  arrCal_DAS_V0[1] = cal_Vref_hi - arrCal_DAS_VperADU[1] * arrCal_DAS_hi_adu[1];
  arrCal_DAS_V0[2] = cal_Vref_hi - arrCal_DAS_VperADU[2] * arrCal_DAS_hi_adu[2];
  arrCal_DAS_V0[3] = cal_Vref_hi - arrCal_DAS_VperADU[3] * arrCal_DAS_hi_adu[3];
  arrCal_DAS_V0[4] = cal_Vref_hi - arrCal_DAS_VperADU[4] * arrCal_DAS_hi_adu[4];

  analogReadResolution(12);
  Serial.begin(ibaudRate);
  while (!Serial);  //waits for the serial port to open- needed for some arduinos

  if (flag_PLX == true) {
    Serial.println("ClearRange,A,5,G,3005");
    Serial.println("LABEL, Time(msec), chan0(V),chan1(V),chan2(V),chan3(V),degC,nptsAve");
    Serial.print("ROW,SET,");
    Serial.println(5);
    Serial.print("COLUMN,SET,");
    Serial.println("A");
  }
  itime2ave_ms = time2ave_sec * 1000L;
  itime0_us = micros(); // start the clock
}

void loop() {

  func_measFast_N_Chan();
  //func_measSlow_N_Chan();
  //func_calibrate_DAS_lo_hi();
  //func_DMM_to_serialMonitor();
  //func_uncalDMM_to_serialMonitor();

}

void func_measSlow_N_Chan() {  // measure up to 4 channels and output the data, averages
  float time_local_sec;
  long icounter1; long icounter2;
  long i; long i2; long i3;
  long itime0_ms; long itime1_ms; long itime2_ms;
  float temperature_degC_local;

  itime0_ms = millis(); //beginning of the measurements
  icounter1 = 0;
  while (1 == 1) {
    icounter1++;
    for (i2 = 0; i2 < nptsChan; i2++) {
      arrV_meas_nChan_slow_v[i2] = 0.;
    }
    itime1_ms = millis();
    icounter2 = 0;
    while (millis() < (itime0_ms + icounter1 * itime2ave_ms)) {
      icounter2++;
      for (i2 = 0; i2 <= nptsChan - 1; i2++) {
        if (i2 < 4) {
          arrV_meas_nChan_slow_v[i2] = arrV_meas_nChan_slow_v[i2] + arrCal_DAS_V0[i2] + (analog.read(i2) * 1.) * arrCal_DAS_VperADU[i2];
        }
        else {
          arrV_meas_nChan_slow_v[i2] = arrV_meas_nChan_slow_v[i2] + arrCal_DAS_V0[i2] + (analogRead(i2) * 1.) * arrCal_DAS_VperADU[i2];
        }
      }
    }
    itime2_ms = millis();
    time_local_sec = ((itime1_ms - itime0_ms) * 1. + (itime2_ms * 1. - itime1_ms * 1.) / 2.) / 1.e3;
    if (flag_PLX == true) {
      Serial.print("DATA,");
    }
    Serial.print(time_local_sec, 3);
    for (i2 = 0; i2 < nptsChan; i2++) {
      Serial.print(",");
      arrV_meas_nChan_slow_v[i2] = arrV_meas_nChan_slow_v[i2] / (icounter2 * 1.);
      if (i2 < 4) {
        Serial.print(arrV_meas_nChan_slow_v[i2], 6);
      }
      else {
        arrV_meas_nChan_slow_v[i2] = (arrV_meas_nChan_slow_v[i2] - 0.5) * 100.;
        Serial.print(arrV_meas_nChan_slow_v[i2], 2);
      }
    }
    Serial.print(",");
    Serial.println(icounter2);
  }
}

void func_measFast_N_Chan() {
  long i; long i2; long i3;
  if (iArray_counter == 0) {
    iArray_counter = 1;
    for (i = 1; i <= npts2meas; i++) {
      for (i2 = 0; i2 < nptsChan; i2++) {
        arrV_meas_nChan_adu[i2][i] = 0.;
      }
    }
    itime0_us = micros();
    for ( i = 1; i <= npts2meas; i++) {
      arrTime_us[i] = (micros() - itime0_us + 10 * npts2ave * 4);
      for (i3 = 1; i3 <= npts2ave; i3++) {
        for (i2 = 0; i2 < nptsChan; i2++) {
          arrV_meas_nChan_adu[i2][i] = analog.read(i2) * 1. + arrV_meas_nChan_adu[i2][i];
        }
      }
    }
    if (flag_PLX == true) {
      Serial.println("ClearRange,A,5,E,1005");
      Serial.print("ROW,SET,");
      Serial.println(5);
      Serial.print("COLUMN,SET,");
      Serial.println("A");
      for ( i = 1; i <= npts2meas; i++) {
        Serial.print("DATA,");
        Serial.print((arrTime_us[i] * 1.) / 1.e3, 4);
        for (i2 = 0; i2 < nptsChan; i2++) {
          Serial.print(",");
          float voltage_meas_local = arrCal_DAS_V0[i2] + (arrV_meas_nChan_adu[i2][i] * 1.) / (npts2ave * 1.) * arrCal_DAS_VperADU[i2];
          Serial.print(voltage_meas_local, 7);
        }
        Serial.println("");
      }
    }
  }
}


void func_DMM_to_serialMonitor() {
  // reads voltage on each channel
  long nptsAve_local = 10000;
  long i; long i2;
  for (i = 0; i <= nptsChan - 1; i++) {
    arrV_meas_nChan_DMM_v[i] = 0.;
    Serial.println("");
  }
  Serial.println("attach a voltage to the inputs");
  for ( i = 1; i <= nptsAve_local; i++) {
    for (i2 = 0; i2 < nptsChan; i2++) {
      if (i2 < 4) {
        arrV_meas_nChan_DMM_v[i2] = arrV_meas_nChan_DMM_v[i2] + arrCal_DAS_V0[i2] + (analog.read(i2) * 1.) * arrCal_DAS_VperADU[i2];
      }
      else {
        arrV_meas_nChan_DMM_v[i2] = arrV_meas_nChan_DMM_v[i2] + arrCal_DAS_V0[i2] + (analogRead(i2) * 1.) * arrCal_DAS_VperADU[i2];
      }
    }
  }
  for (i2 = 0; i2 < nptsChan; i2++) {
    arrV_meas_nChan_DMM_v[i2] = arrV_meas_nChan_DMM_v[i2] / (nptsAve_local);
    Serial.print("Value for channel "); Serial.print(i2); Serial.print(": "); Serial.println(arrV_meas_nChan_DMM_v[i2], 6);
  }
}

void func_uncalDMM_to_serialMonitor() {
  // reads voltage on each channel
  long nptsAve_local = 10000;

  long i; long i2;
  for (i = 0; i <= nptsChan - 1; i++) {
    arrV_meas_nChan_DMM_v[i] = 0.;
    Serial.println("");
  }
  Serial.println("attach a voltage to the inputs");
  for ( i = 1; i <= nptsAve_local; i++) {
    for (i2 = 0; i2 < nptsChan; i2++) {
      arrV_meas_nChan_DMM_v[i2] = arrV_meas_nChan_DMM_v[i2] + ((analog.read(i2) * 1.) - 32767. ) * 10. / 65535. ;
    }
  }
  for (i2 = 0; i2 < nptsChan; i2++) {
    arrV_meas_nChan_DMM_v[i2] = arrV_meas_nChan_DMM_v[i2] / (nptsAve_local);
    Serial.print("Value for channel "); Serial.print(i2); Serial.print(": "); Serial.println(arrV_meas_nChan_DMM_v[i2], 6);
  }
}

void func_calibrate_DAS_lo_hi() {
  long nptsAve_local = 10000;
  long i; long i2;
  for (i = 0; i <= nptsChan - 1; i++) {
    iCal_chan_adu[i] = 0;
    Serial.println("");
  }
  Serial.println("attach ref to inputs");
  for ( i = 1; i <= nptsAve_local; i++) {
    for (i2 = 0; i2 < nptsChan; i2++) {
      if (i2 < 4) {
        iCal_chan_adu[i2] = analog.read(i2) + iCal_chan_adu[i2];
      }
      else {
        iCal_chan_adu[i2] = analogRead(i2) + iCal_chan_adu[i2];
      }
    }
  }

  for (i2 = 0; i2 < nptsChan; i2++) {
    iCal_chan_adu[i2] = iCal_chan_adu[i2] / nptsAve_local;
    Serial.print("ref for channel "); Serial.print(i2); Serial.print(": "); Serial.println(iCal_chan_adu[i2]);
  }
}




